@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2010-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2010-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-kajongg is free software you can redistribute it and/or modify
+Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 2 of the License, or
 (at your option) any later version.
@@ -20,27 +20,27 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 from twisted.internet.defer import Deferred, succeed
 
-from PyQt4.QtCore import QPropertyAnimation, QParallelAnimationGroup, \
+from qt import QPropertyAnimation, QParallelAnimationGroup, \
     QAbstractAnimation, QEasingCurve, QVariant
 
-from common import Internal, Preferences, Debug, isAlive
-from util import logDebug
+from common import Internal, Debug, isAlive
+from log import logDebug
 
 class Animation(QPropertyAnimation):
     """a Qt4 animation with helper methods"""
 
     nextAnimations = []
 
-    def __init__(self, target, propName, endValue, parent=None):
-        QPropertyAnimation.__init__(self, target, propName, parent)
+    def __init__(self, uiTile, propName, endValue, parent=None):
+        QPropertyAnimation.__init__(self, uiTile, propName, parent)
         QPropertyAnimation.setEndValue(self, endValue)
-        duration = Preferences.animationDuration()
+        duration = Internal.Preferences.animationDuration()
         self.setDuration(duration)
         self.setEasingCurve(QEasingCurve.InOutQuad)
-        target.queuedAnimations.append(self)
+        uiTile.queuedAnimations.append(self)
         Animation.nextAnimations.append(self)
-        if target.tile in Debug.animation:
-            oldAnimation = target.activeAnimation.get(propName, None)
+        if uiTile.tile in Debug.animation:
+            oldAnimation = uiTile.activeAnimation.get(propName, None)
             if isAlive(oldAnimation):
                 logDebug('new animation %s (after %s is done)' % (self, oldAnimation.ident()))
             else:
@@ -48,11 +48,11 @@ class Animation(QPropertyAnimation):
 
     def setEndValue(self, endValue):
         """wrapper with debugging code"""
-        tile = self.targetObject()
-        if tile.tile in Debug.animation:
+        uiTile = self.targetObject()
+        if uiTile.tile in Debug.animation:
             pName = self.pName()
             logDebug('%s: change endValue for %s: %s->%s  %s' % (self.ident(), pName, self.formatValue(self.endValue()),
-                    self.formatValue(endValue), tile))
+                    self.formatValue(endValue), uiTile))
         QPropertyAnimation.setEndValue(self, endValue)
 
     def ident(self):
@@ -69,6 +69,8 @@ class Animation(QPropertyAnimation):
 
     def unpackValue(self, qvariant):
         """get the wanted value from the QVariant"""
+        if not isinstance(qvariant, QVariant):
+            return qvariant # is already autoconverted
         pName = self.pName()
         if pName == 'pos':
             return qvariant.toPointF()
@@ -95,9 +97,8 @@ class Animation(QPropertyAnimation):
 
     def __str__(self):
         """for debug messages"""
-        pName = self.pName()
-        tile = self.targetObject()
-        return '%s: %s->%s for %s' % (self.ident(), pName, self.formatValue(self.endValue()), tile)
+        return '%s: %s->%s for %s' % (self.ident(), self.pName(),
+            self.formatValue(self.endValue()), self.targetObject())
 
 class ParallelAnimationGroup(QParallelAnimationGroup):
     """override __init__"""
@@ -130,37 +131,35 @@ class ParallelAnimationGroup(QParallelAnimationGroup):
             # periodically check if the board still exists.
             # if not (game end), we do not want to go on
             for animation in self.animations:
-                tile = animation.targetObject()
-                if not isAlive(tile.board):
-                    tile.clearActiveAnimation(animation)
+                uiTile = animation.targetObject()
+                if not isAlive(uiTile.board):
+                    uiTile.clearActiveAnimation(animation)
                     self.removeAnimation(animation)
         QParallelAnimationGroup.updateCurrentTime(self, value)
 
     def start(self, dummyResults='DIREKT'):
         """start the animation, returning its deferred"""
         assert self.state() != QAbstractAnimation.Running
-        tiles = set()
         for animation in self.animations:
-            tile = animation.targetObject()
-            self.debug |= tile.tile in Debug.animation
-            tiles.add(tile)
-            tile.setActiveAnimation(animation)
+            uiTile = animation.targetObject()
+            self.debug |= uiTile.tile in Debug.animation
+            uiTile.setActiveAnimation(animation)
             self.addAnimation(animation)
             propName = animation.pName()
-            animation.setStartValue(tile.getValue(propName))
+            animation.setStartValue(uiTile.getValue(propName))
             if propName == 'rotation':
                 # change direction if that makes the difference smaller
                 endValue = animation.unpackEndValue()
-                currValue = tile.rotation
+                currValue = uiTile.rotation
                 if endValue - currValue > 180:
                     animation.setStartValue(currValue + 360)
                 if currValue - endValue > 180:
                     animation.setStartValue(currValue - 360)
-        for tile in tiles:
-            tile.setDrawingOrder()
+        for animation in self.animations:
+            animation.targetObject().setDrawingOrder()
         self.finished.connect(self.allFinished)
-        scene = Internal.field.centralScene
-        scene.disableFocusRect = True
+        scene = Internal.scene
+        scene.focusRect.hide()
         QParallelAnimationGroup.start(self, QAbstractAnimation.DeleteWhenStopped)
         if self.debug:
             logDebug('Animation group %d started (%s)' % (
@@ -188,31 +187,31 @@ class ParallelAnimationGroup(QParallelAnimationGroup):
     def fixAllBoards(self):
         """set correct drawing order for all moved tiles"""
         for animation in self.children():
-            tile = animation.targetObject()
-            if tile:
-                tile.clearActiveAnimation(animation)
-        scene = Internal.field.centralScene
-        scene.disableFocusRect = False
+            uiTile = animation.targetObject()
+            if uiTile:
+                uiTile.clearActiveAnimation(animation)
+        if Internal.scene:
+            Internal.scene.focusRect.refresh()
         return
 
 class Animated(object):
     """a helper class for moving tiles with or without animation"""
     def __init__(self, animateMe=True):
-        if Preferences:
+        if Internal.Preferences:
             self.__animateMe = animateMe
-            self.prevAnimationSpeed = Preferences.animationSpeed
+            self.prevAnimationSpeed = Internal.Preferences.animationSpeed
             if not animateMe:
-                Preferences.animationSpeed = 99
+                Internal.Preferences.animationSpeed = 99
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_value, trback):
         """reset previous animation speed"""
-        if Preferences:
+        if Internal.Preferences:
             if not self.__animateMe:
                 animate()
-                Preferences.animationSpeed = self.prevAnimationSpeed
+                Internal.Preferences.animationSpeed = self.prevAnimationSpeed
 
 
 def afterCurrentAnimationDo(callback, *args, **kwargs):
@@ -237,10 +236,9 @@ def animate():
              - there are too many animations in the group so it would be too slow
     """
     if Animation.nextAnimations:
-        field = Internal.field
-        shortcutMe = (field is None
-                or field.centralView.dragObject
-                or Preferences.animationSpeed == 99
+        shortcutMe = (Internal.scene is None
+                or Internal.mainWindow.centralView.dragObject
+                or Internal.Preferences.animationSpeed == 99
                 or len(Animation.nextAnimations) > 1000)
                 # change 1000 to 100 if we do not want to animate shuffling and initial deal
         if not shortcutMe:
@@ -252,11 +250,10 @@ def animate():
             shortcutMe = duration == 0
         if shortcutMe:
             for animation in Animation.nextAnimations:
-                tile = animation.targetObject()
-                tile.shortcutAnimation(animation)
+                animation.targetObject().shortcutAnimation(animation)
             Animation.nextAnimations = []
-            scene = Internal.field.centralScene
-            scene.disableFocusRect = False
+            if Internal.scene:
+                Internal.scene.focusRect.refresh()
             return succeed(None)
         else:
             return ParallelAnimationGroup().deferred
