@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 """
-    Copyright (C) 2010-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+    Copyright (C) 2010-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
     partially based on C++ code from:
     Copyright (C) 2006 Mauricio Piacentini <mauricio@tabuleiro.com>
@@ -21,13 +22,12 @@
 
 import os
 
-from PyQt4.QtCore import QVariant
-from PyQt4.QtGui import QComboBox, QTableView, QSizePolicy, QAbstractItemView
+from qt import uic, QVariant, variantValue
+from qt import QComboBox, QTableView, QSizePolicy, QAbstractItemView
 
-from kde import KStandardDirs, KConfig, KConfigGroup
-from PyQt4 import uic
+from kde import KStandardDirs, KIcon
 
-from util import m18n
+from log import m18n
 
 def loadUi(base):
     """load the ui file for class base, deriving the file name from the class name"""
@@ -58,8 +58,6 @@ class MJTableView(QTableView):
 
 class ListComboBox(QComboBox):
     """easy to use with a python list. The elements must have an attribute 'name'."""
-    # pylint: disable=incomplete-protocol
-    # pylint wants __setitem__, __getitem__ etc for a perfect container
     def __init__(self, items=None, parent=None):
         QComboBox.__init__(self, parent)
         self.items = items
@@ -67,7 +65,7 @@ class ListComboBox(QComboBox):
     @property
     def items(self):
         """combo box items"""
-        return [self.itemData(idx).toPyObject() for idx in range(self.count())]
+        return [variantValue(self.itemData(idx)) for idx in range(self.count())]
 
     @items.setter
     def items(self, items):
@@ -91,7 +89,7 @@ class ListComboBox(QComboBox):
     @property
     def current(self):
         """current item"""
-        return self.itemData(self.currentIndex()).toPyObject()
+        return variantValue(self.itemData(self.currentIndex()))
 
     @current.setter
     def current(self, item):
@@ -101,7 +99,24 @@ class ListComboBox(QComboBox):
             raise Exception('%s not found in ListComboBox' % item.name)
         self.setCurrentIndex(newIdx)
 
-def konfigGroup(path, groupName):
-    """returns access to a group of config options"""
-    config = KConfig(path, KConfig.SimpleConfig)
-    return config, KConfigGroup(config.group(groupName))
+class Painter(object):
+    """a helper class for painting: saves/restores painter"""
+    def __init__(self, painter):
+        """painter is the painter to be saved/restored"""
+        self.painter = painter
+
+    def __enter__(self):
+        self.painter.save()
+        return self
+
+    def __exit__(self, exc_type, exc_value, trback):
+        """now check time passed"""
+        self.painter.restore()
+
+def decorateWindow(window, name=None):
+    """standard Kajongg window title and icon"""
+    if name:
+        window.setWindowTitle(u'{} – {}'.format(name, m18n('Kajongg')))
+    else:
+        window.setWindowTitle(m18n('Kajongg'))
+    window.setWindowIcon(KIcon('kajongg'))
