@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-kajongg is free software you can redistribute it and/or modify
+Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 2 of the License, or
 (at your option) any later version.
@@ -18,53 +18,227 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-import socket, subprocess, time, datetime, os, sys
-import re
+import socket, subprocess, datetime, os, sys
+from itertools import chain
 
 from twisted.spread import pb
 from twisted.cred import credentials
 from twisted.internet.defer import CancelledError
+from twisted.internet.task import deferLater
+from twisted.internet.error import ConnectionRefusedError, TimeoutError, ConnectionLost, DNSLookupError, ConnectError
 from twisted.python.failure import Failure
 
-from PyQt4.QtGui import QDialog, QDialogButtonBox, QVBoxLayout, \
+from qt import QDialog, QDialogButtonBox, QVBoxLayout, \
     QLabel, QComboBox, QLineEdit, QFormLayout, \
-    QSizePolicy
+    QSizePolicy, QWidget
 
-from kde import DeferredDialog, QuestionYesNo, KDialogButtonBox, KUser, \
-    MustChooseDialog
+from kde import KUser, KDialog, KDialogButtonBox, appdataDir, socketName
+from dialogs import DeferredDialog, QuestionYesNo, MustChooseKDialog
 
-from util import m18n, m18nc, logWarning, logException, socketName, english, \
-    appdataDir, logInfo, logDebug, removeIfExists, which
-from util import SERVERMARK
-from common import Internal, Options, SingleshotOptions, Internal, Debug
+from log import logWarning, logException, logInfo, logDebug, m18n, m18nc, SERVERMARK
+from util import removeIfExists, which
+from common import Internal, Options, SingleshotOptions, Internal, Debug, isAlive, english
 from game import Players
-from query import Transaction, Query
+from query import Query
 from statesaver import StateSaver
 
-from guiutil import ListComboBox
+from guiutil import ListComboBox, decorateWindow
 from rule import Ruleset
 
 class LoginAborted(Exception):
     """the user aborted the login"""
     pass
 
+class Url(str):
+    """holds connection related attributes: host, port, socketname"""
+    # pylint: disable=too-many-public-methods
+    def __init__(self, url):
+        self.host = None
+        self.port = None
+        if url:
+            urlParts = url.split(':')
+            self.host = urlParts[0]
+            if english(self.host) == Query.localServerName:
+                self.host = '127.0.0.1'
+            if len(urlParts) > 1:
+                self.port = int(urlParts[1])
+        else:
+            url = self.host
+            if self.port:
+                url += ':{}'.format(self.port)
+        str.__init__(self, url)
+        if Options.socket and os.name == 'nt':
+            self.port = int(Options.socket)
+        if self.port is None and self.isLocalHost and not self.useSocket:
+            self.port = self.__findFreePort()
+        if self.port is None and not self.isLocalHost:
+            self.port = Options.defaultPort()
+        if Debug.connections:
+            logDebug(repr(self))
+
+        if Internal.reactor is None:
+            import qt4reactor
+            qt4reactor.install()
+            from twisted.internet import reactor
+            reactor.runReturn(installSignalHandlers=False)
+            Internal.reactor = reactor
+            if Debug.quit:
+                logDebug('Installed qt4reactor')
+
+    def __repr__(self):
+        """show all info"""
+        if self.useSocket:
+            return 'Url({} socket={})'.format(self, socketName())
+        else:
+            return 'Url({} host={} port={})'.format(self, self.host, self.port)
+
+    @property
+    def useSocket(self):
+        """do we use socket for current host?"""
+        return self.host == '127.0.0.1' and os.name != 'nt'
+
+    @property
+    def isLocalGame(self):
+        """Are we playing a local game not needing the network?"""
+        return self.host == '127.0.0.1'
+
+    @property
+    def isLocalHost(self):
+        """do server and client run on the same host?"""
+        return self.host in ('127.0.0.1', 'localhost')
+
+    def __findFreePort(self):
+        """find an unused port on the current system.
+        used when we want to start a local server on windows"""
+        assert self.isLocalHost
+        for port in chain([Options.defaultPort()], range(2000, 19000)):
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            sock.settimeout(1)
+            try:
+                sock.connect((self.host, port))
+                sock.close()
+            except socket.error:
+                return port
+        logException('cannot find a free port')
+
+    def startServer(self, result, waiting=0):
+        """make sure we have a running local server or network connectivity"""
+        if self.isLocalHost:
+            # just wait for that server to appear
+            if self.__serverListening():
+                return result
+            else:
+                if waiting == 0:
+                    self.__startLocalServer()
+                elif waiting > 30:
+                    logDebug('Game %s: Server %s not available after 30 seconds, aborting' % (
+                        SingleshotOptions.game, self))
+                    raise CancelledError
+                return deferLater(Internal.reactor, 1, self.startServer, result, waiting+1)
+        elif which('XXqdbus'):
+            # TODO: use twisted process because we must have a timeout. If the qdbus service
+            # does not answer, qdbus waits for 25 seconds.
+            # the state of QtDBus is unclear to me.
+            # riverbank.computing says module dbus is deprecated
+            # for Python 3. And Ubuntu has no package with
+            # QtDBus. So we use good old subprocess.
+            stdoutdata, stderrdata = subprocess.Popen(['qdbus',
+                'org.kde.kded',
+                '/modules/networkstatus',
+                'org.kde.Solid.Networking.status'], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
+            stdoutdata = stdoutdata.strip()
+            stderrdata = stderrdata.strip()
+            if stderrdata == '' and stdoutdata != '4':
+                # pylint: disable=nonstandard-exception
+                raise ConnectError()
+            # if we have stderrdata, qdbus probably does not provide the service we want, so ignore it
+        return result
+
+    def __startLocalServer(self):
+        """start a local server"""
+        # pylint: disable=too-many-branches
+        try:
+            if sys.argv[0].endswith('kajongg.py'):
+                tryServer = sys.argv[0].replace('.py', 'server.py')
+                if os.path.exists(tryServer):
+                    args = ['python', tryServer]
+            elif sys.argv[0].endswith('kajongg.pyw'):
+                tryServer = sys.argv[0].replace('.pyw', 'server.py')
+                if os.path.exists(tryServer):
+                    args = ['python', tryServer]
+            elif sys.argv[0].endswith('kajongg.exe'):
+                tryServer = sys.argv[0].replace('.exe', 'server.exe')
+                if os.path.exists(tryServer):
+                    args = [tryServer]
+            else:
+                args = ['kajonggserver']
+            if self.useSocket or os.name == 'nt': # for nt --socket tells the server to bind to 127.0.0.1
+                args.append('--socket=%s' % socketName())
+                if removeIfExists(socketName()):
+                    logInfo(m18n('removed stale socket <filename>%1</filename>', socketName()))
+            if not self.useSocket:
+                args.append('--port=%d' % self.port)
+            if self.isLocalGame:
+                args.append('--db={}'.format(os.path.normpath(appdataDir() + 'local.db')))
+            if Debug.argString:
+                args.append('--debug=%s' % Debug.argString)
+            if os.name == 'nt':
+                startupinfo = subprocess.STARTUPINFO()
+                startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+            else:
+                startupinfo = None
+            process = subprocess.Popen(args, startupinfo=startupinfo) # , shell=os.name == 'nt')
+            if Debug.connections:
+                logDebug(m18n('started the local kajongg server: pid=<numid>%1</numid> %2',
+                    process.pid, ' '.join(args)))
+        except OSError as exc:
+            logException(exc)
+
+    def __serverListening(self):
+        """is the expected server listening?"""
+        if self.useSocket:
+            proto = socket.AF_UNIX
+            param = socketName()
+            if not os.path.exists(param):
+                return False
+        else:
+            proto = socket.AF_INET
+            param = (self.host, self.port)
+        sock = socket.socket(proto, socket.SOCK_STREAM)
+        sock.settimeout(1)
+        try:
+            sock.connect(param)
+            sock.close()
+            return True
+        except socket.error:
+            return False
+
+    def connect(self, factory):
+        """returns a twisted connector"""
+        if self.useSocket:
+            return Internal.reactor.connectUNIX(socketName(), factory, timeout=5)
+        else:
+            host = self.host
+            return Internal.reactor.connectTCP(host, self.port, factory, timeout=5)
+
 class LoginDlg(QDialog):
     """login dialog for server"""
     def __init__(self):
         """self.servers is a list of tuples containing server and last playername"""
         QDialog.__init__(self, None)
-        self.setWindowTitle(m18n('Login') + ' - Kajongg')
+        decorateWindow(self, m18n('Login'))
         self.setupUi()
 
-        localName = m18nc('kajongg name for local game server', Query.localServerName)
+        localName = m18n(Query.localServerName)
         self.servers = Query('select url,lastname from server order by lasttime desc').records
-        servers = [m18nc('kajongg name for local game server', x[0]) for x in self.servers]
+        servers = [m18n(x[0]) for x in self.servers]
         # the first server combobox item should be default: either the last used server
         # or localName for autoPlay
         if localName not in servers:
             servers.append(localName)
         if 'kajongg.org' not in servers:
             servers.append('kajongg.org')
+        if Internal.autoPlay:
             demoHost = Options.host or localName
             if demoHost in servers:
                 servers.remove(demoHost)  # we want a unique list, it will be re-used for all following games
@@ -79,8 +253,8 @@ class LoginDlg(QDialog):
         StateSaver(self)
 
     def returns(self, dummyButton=None):
-        """maybe we should return an class ServerConnection"""
-        return (self.useSocket, self.url, self.username, self.password, self.__defineRuleset())
+        """login data returned by this dialog"""
+        return (Url(self.url), self.username, self.password, self.__defineRuleset())
 
     def setupUi(self):
         """create all Ui elements but do not fill them"""
@@ -110,11 +284,12 @@ class LoginDlg(QDialog):
         pol = QSizePolicy()
         pol.setHorizontalPolicy(QSizePolicy.Expanding)
         self.cbUser.setSizePolicy(pol)
+        self.__port = None
 
     def serverChanged(self, dummyText=None):
         """the user selected a different server"""
         records = Query('select player.name from player, passwords '
-                'where passwords.url=? and passwords.player = player.id', list([self.url])).records
+                'where passwords.url=? and passwords.player = player.id', (self.url,)).records
         players = list(x[0] for x in records)
         preferPlayer = Options.player
         if preferPlayer:
@@ -132,7 +307,7 @@ class LoginDlg(QDialog):
                 userIdx = self.cbUser.findText(userNames[0])
                 if userIdx >= 0:
                     self.cbUser.setCurrentIndex(userIdx)
-        showPW = self.url != Query.localServerName
+        showPW = not Url(self.url).isLocalHost
         self.grid.labelForField(self.edPassword).setVisible(showPW)
         self.edPassword.setVisible(showPW)
         self.grid.labelForField(self.cbRuleset).setVisible(not showPW and not Options.ruleset)
@@ -172,24 +347,6 @@ class LoginDlg(QDialog):
         """abstracts the url of the dialog"""
         return english(unicode(self.cbServer.currentText()))
 
-    @property
-    def host(self):
-        """abstracts the host of the dialog"""
-        return self.url.partition(':')[0]
-
-    @property
-    def useSocket(self):
-        """do we use socket for current host?"""
-        return self.host == Query.localServerName
-
-    @property
-    def port(self):
-        """abstracts the port of the dialog"""
-        try:
-            return int(self.url.partition(':')[2])
-        except ValueError:
-            return Options.defaultPort()
-
     @property
     def username(self):
         """abstracts the username of the dialog"""
@@ -205,18 +362,15 @@ class LoginDlg(QDialog):
         """abstracts the password of the dialog"""
         self.edPassword.setText(password)
 
-class AddUserDialog(MustChooseDialog):
+class AddUserDialog(MustChooseKDialog):
     """add a user account on a server: This dialog asks for the needed attributes"""
     # pylint: disable=too-many-instance-attributes
 
     def __init__(self, url, username, password):
-        MustChooseDialog.__init__(self, None)
-        self.setWindowTitle(m18n('Create User Account') + ' - Kajongg')
-        self.buttonBox = KDialogButtonBox(self)
-        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
-        self.buttonBox.accepted.connect(self.accept)
-        self.buttonBox.rejected.connect(self.reject)
-        vbox = QVBoxLayout(self)
+        MustChooseKDialog.__init__(self)
+        decorateWindow(self, m18n('Create User Account'))
+        self.setButtons(KDialog.ButtonCode(KDialog.Ok | KDialog.Cancel))
+        vbox = QVBoxLayout()
         grid = QFormLayout()
         self.lbServer = QLabel()
         self.lbServer.setText(url)
@@ -230,7 +384,9 @@ class AddUserDialog(MustChooseDialog):
         self.edPassword2.setEchoMode(QLineEdit.PasswordEchoOnEdit)
         grid.addRow(m18n('Repeat password:'), self.edPassword2)
         vbox.addLayout(grid)
-        vbox.addWidget(self.buttonBox)
+        widget = QWidget(self)
+        widget.setLayout(vbox)
+        self.setMainWidget(widget)
         pol = QSizePolicy()
         pol.setHorizontalPolicy(QSizePolicy.Expanding)
         self.lbUser.setSizePolicy(pol)
@@ -250,7 +406,7 @@ class AddUserDialog(MustChooseDialog):
     def validate(self):
         """does the dialog hold valid data?"""
         equal = self.edPassword.size() and self.edPassword.text() == self.edPassword2.text()
-        self.buttonBox.button(QDialogButtonBox.Ok).setEnabled(equal)
+        self.button(KDialog.Ok).setEnabled(equal)
 
     @property
     def username(self):
@@ -279,35 +435,58 @@ class Connection(object):
         self.client = client
         self.perspective = None
         self.connector = None
-        self.useSocket = False
         self.url = None
         self.username = None
         self.password = None
-        self.ruleset = None
+        self.__ruleset = None
         self.dlg = LoginDlg()
 
+    @property
+    def ruleset(self):
+        """reader"""
+        return self.__ruleset
+
+    @ruleset.setter
+    def ruleset(self, value):
+        """save changed ruleset as last used ruleset for this server"""
+        if  self.__ruleset != value:
+            self.__ruleset = value
+            if value:
+                def write():
+                    """write to database, returns 1 for success"""
+                    return Query('update server set lastruleset=? where url=?', (value.rulesetId, self.url))
+                value.save()     # make sure we have a valid rulesetId for predefined rulesets
+                if not write():
+                    self.__updateServerInfoInDatabase()
+                    write()
+
     def login(self):
         """to be called from HumanClient"""
-        result = DeferredDialog(self.dlg).addCallback(self.__haveLoginData
-            ).addCallbacks(self.assertConnectivity, self._loginReallyFailed
-            ).addCallbacks(self.loginToServer, self._loginReallyFailed
-            ).addCallback(self.loggedIn)
+        result = DeferredDialog(self.dlg).addCallback(self.__haveLoginData)
+        result.addCallback(self.__checkExistingConnections)
+        result.addCallback(self.__startServer)
+        result.addCallback(self.__loginToServer)
+        result.addCallback(self.loggedIn)
+        result.addErrback(self._loginReallyFailed)
         if Internal.autoPlay or SingleshotOptions.table or SingleshotOptions.join:
             result.clicked()
         return result
 
-    def loginToServer(self, dummy=None):
-        """login to server"""
-        return self.loginCommand(self.username).addErrback(self._loginFailed)
-
     def __haveLoginData(self, arguments):
         """user entered login data, now try to login to server"""
-        if self.url == 'localhost':
+        self.url, self.username, self.password, self.ruleset = arguments
+        if self.url.isLocalHost:
             # we have localhost if we play a Local Game: client and server are identical,
             # we have no security concerns about creating a new account
             Players.createIfUnknown(unicode(self.dlg.cbUser.currentText()))
-        self.useSocket, self.url, self.username, self.password, self.ruleset = arguments
-        self.__checkExistingConnections()
+
+    def __startServer(self, result):
+        """if needed"""
+        return self.url.startServer(result)
+
+    def __loginToServer(self, dummy=None):
+        """login to server"""
+        return self.loginCommand(self.username).addErrback(self._loginFailed)
 
     def loggedIn(self, perspective):
         """successful login on server"""
@@ -322,23 +501,22 @@ class Connection(object):
     def __updateServerInfoInDatabase(self):
         """we are online. Update table server."""
         lasttime = datetime.datetime.now().replace(microsecond=0).isoformat()
-        url = english(self.url) # use unique name for Local Game
-        with Transaction():
-            serverKnown = Query('update server set lastname=?,lasttime=?,lastruleset=? where url=?',
-                list([self.username, lasttime, self.ruleset.rulesetId, url])).rowcount() == 1
+        with Internal.db:
+            serverKnown = Query('update server set lastname=?,lasttime=? where url=?',
+                (self.username, lasttime, self.url)).rowcount() == 1
             if not serverKnown:
-                Query('insert into server(url,lastname,lasttime,lastruleset) values(?,?,?,?)',
-                    list([url, self.username, lasttime, self.ruleset.rulesetId]))
+                Query('insert into server(url,lastname,lasttime) values(?,?,?)',
+                    (self.url, self.username, lasttime))
         # needed if the server knows our name but our local data base does not:
         Players.createIfUnknown(self.username)
         playerId = Players.allIds[self.username]
-        with Transaction():
+        with Internal.db:
             if Query('update passwords set password=? where url=? and player=?',
-                list([self.password, url, playerId])).rowcount() == 0:
+                (self.password, self.url, playerId)).rowcount() == 0:
                 Query('insert into passwords(url,player,password) values(?,?,?)',
-                    list([url, playerId, self.password]))
+                    (self.url, playerId, self.password))
 
-    def __checkExistingConnections(self):
+    def __checkExistingConnections(self, dummy=None):
         """do we already have a connection to the wanted URL?"""
         for client in self.client.humanClients:
             if client.connection and client.connection.url == self.url:
@@ -346,112 +524,11 @@ class Connection(object):
                 client.tableList.activateWindow()
                 raise CancelledError
 
-    @staticmethod
-    def findFreePort():
-        """find an unused port on the current system.
-        used when we want to start a local server on windows"""
-        for port in range(2000, 9000):
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.settimeout(1)
-            try:
-                sock.connect(('127.0.0.1', port))
-            except socket.error:
-                return port
-        logException('cannot find a free port')
-
-    def serverListening(self):
-        """is somebody listening on that port?"""
-        if self.useSocket and os.name != 'nt':
-            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-            sock.settimeout(1)
-            try:
-                sock.connect(socketName())
-            except socket.error as exception:
-                if os.path.exists(socketName()):
-                    # try again, avoiding a race
-                    try:
-                        sock.connect(socketName())
-                    except socket.error as exception:
-                        if removeIfExists(socketName()):
-                            logInfo(m18n('removed stale socket <filename>%1</filename>', socketName()))
-                        logInfo('socket error:%s' % str(exception))
-                        return False
-                    else:
-                        return True
-            else:
-                return True
-        else:
-            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            sock.settimeout(1)
-            try:
-                sock.connect((self.dlg.host, self.dlg.port))
-            except socket.error:
-                return False
-            else:
-                return True
-
-    def assertConnectivity(self, result):
-        """make sure we have a running local server or network connectivity"""
-        if self.useSocket or self.dlg.url in ('localhost', '127.0.0.1'):
-            if not self.serverListening():
-                if os.name == 'nt':
-                    port = self.findFreePort()
-                else:
-                    port = None
-                self.startLocalServer(port)
-                # give the server up to 5 seconds time to start
-                for loop in range(50):
-                    if self.serverListening():
-                        break
-                    time.sleep(0.1)
-        elif which('qdbus'):
-            # the state of QtDBus is unclear to me.
-            # riverbank.computing says module dbus is deprecated
-            # for Python 3. And Ubuntu has no package with
-            # PyQt4.QtDBus. So we use good old subprocess.
-            answer = subprocess.Popen(['qdbus',
-                'org.kde.kded',
-                '/modules/networkstatus',
-                'org.kde.Solid.Networking.status'], stdout=subprocess.PIPE).communicate()[0].strip()
-            if answer != '4':
-                # pylint: disable=nonstandard-exception
-                raise Failure(m18n('You have no network connectivity: %1', answer))
-        return result
-
-    def startLocalServer(self, port):
-        """start a local server"""
-        try:
-            args = ['kajonggserver'] # the default
-            if sys.argv[0].endswith('kajongg.py'):
-                tryServer = sys.argv[0].replace('.py', 'server.py')
-                if os.path.exists(tryServer):
-                    args = ['python', tryServer]
-            if self.useSocket or os.name == 'nt':
-                args.append('--local')
-            if port:
-                args.append('--port=%d' % port)
-            if self.useSocket:
-                args.append('--db=%slocal.db' % appdataDir())
-            if Debug.argString:
-                args.append('--debug=%s' % Debug.argString)
-            if Options.socket:
-                args.append('--socket=%s' % Options.socket)
-            process = subprocess.Popen(args, shell=os.name=='nt')
-            if Debug.connections:
-                logDebug(m18n('started the local kajongg server: pid=<numid>%1</numid> %2',
-                    process.pid, ' '.join(args)))
-        except OSError as exc:
-            logException(exc)
-
     def loginCommand(self, username):
         """send a login command to server. That might be a normal login
         or adduser/deluser/change passwd encoded in the username"""
         factory = pb.PBClientFactory()
-        reactor = Internal.reactor
-        if self.useSocket and os.name != 'nt':
-            self.connector = reactor.connectUNIX(socketName(), factory, timeout=2)
-        else:
-            self.connector = reactor.connectTCP(self.dlg.host, self.dlg.port, factory, timeout=5)
+        self.connector = self.url.connect(factory)
         utf8Password = self.dlg.password.encode('utf-8')
         utf8Username = username.encode('utf-8')
         cred = credentials.UsernamePassword(utf8Username, utf8Password)
@@ -460,42 +537,15 @@ class Connection(object):
     def __adduser(self):
         """create a user account"""
         assert self.url is not None
-        if self.dlg.host != Query.localServerName:
-            if not AddUserDialog(self.dlg.url,
+        if not self.url.isLocalHost:
+            if not AddUserDialog(self.url,
                 self.dlg.username,
                 self.dlg.password).exec_():
-                return
+                raise CancelledError
             Players.createIfUnknown(self.username)
         adduserCmd = SERVERMARK.join(['adduser', self.dlg.username, self.dlg.password])
         return self.loginCommand(adduserCmd)
 
-    def _prettifyErrorMessage(self, failure):
-        """instead of just failure.getErrorMessage(), return something more user friendly.
-        That will be a localized error text, the original english text will be removed"""
-        url = self.url
-        message = failure.getErrorMessage()
-        match = re.search(r".*gaierror\(-\d, '(.*)'.*", message)
-        if not match:
-            match = re.search(r".*ConnectError\('(.*)',\)", message)
-        if not match:
-            match = re.search(r".*ConnectionRefusedError\('(.*)',\)", message)
-        if not match:
-            match = re.search(r".*DNS lookup.*\[Errno -5\] (.*)", message)
-            if match:
-                url = url.split(':')[0] # remove the port
-        # current twisted (version 12.3) returns different messages:
-        if not match:
-            match = re.search(r".*DNS lookup failed: address u'(.*)' not found.*", message)
-            if match:
-                return u'%s: %s' % (match.group(1), m18n('DNS lookup failed, address not found'))
-        if not match:
-            match = re.search(r".*DNS lookup.*\[Errno 110\] (.*)", message)
-        if not match:
-            match = re.search(r".*while connecting: 113: (.*)", message)
-        if match:
-            message = match.group(1).decode('string-escape').decode('string-escape')
-        return u'%s: %s' % (url, message.decode('utf-8'))
-
     def _loginFailed(self, failure):
         """login failed"""
         def answered(result):
@@ -503,33 +553,48 @@ class Connection(object):
             if result:
                 return self.__adduser()
             else:
-                return Failure(CancelledError)
+                return Failure(CancelledError())
         message = failure.getErrorMessage()
         if 'Wrong username' in message:
-            if self.dlg.host == Query.localServerName:
+            if self.url.isLocalHost:
                 return answered(True)
             else:
                 msg = m18nc('USER is not known on SERVER',
-                    '%1 is not known on %2, do you want to open an account?', self.dlg.username, self.dlg.host)
+                    '%1 is not known on %2, do you want to open an account?', self.dlg.username, self.url.host)
                 return QuestionYesNo(msg).addCallback(answered)
         else:
             return self._loginReallyFailed(failure)
 
     def _loginReallyFailed(self, failure):
         """login failed, not fixable by adding missing user"""
-        msg = self._prettifyErrorMessage(failure)
+        msg = None
+        if not isAlive(Internal.mainWindow):
+            raise CancelledError
         if failure.check(CancelledError):
-            # show no warning, just leave
-            return failure
-        if 'Errno 5' in msg:
-            # The server is running but something is wrong with it
-            if self.useSocket and os.name != 'nt':
-                if removeIfExists(socketName()):
-                    logInfo(m18n('removed stale socket <filename>%1</filename>', socketName()))
-                msg += '\n\n\n' + m18n('Please try again')
+            pass
+        elif failure.check(TimeoutError):
+            msg = m18n('Server %1 did not answer', self.url)
+        elif failure.check(ConnectionRefusedError):
+            msg = m18n('Server %1 refused connection', self.url)
+        elif failure.check(ConnectionLost):
+            msg = m18n('Server %1 does not run a kajongg server', self.url)
+        elif failure.check(DNSLookupError):
+            msg = m18n('Address for server %1 cannot be found', self.url)
+        elif failure.check(ConnectError):
+            msg = m18n('Login to server %1 failed: You have no network connection', self.url)
+        else:
+            msg = 'Login to server {} failed: {}/{} Callstack:{}'.format(
+                self.url, failure.value.__class__.__name__, failure.getErrorMessage(),
+                failure.getTraceback())
+        # Maybe the server is running but something is wrong with it
+        if self.url.useSocket:
+            if removeIfExists(socketName()):
+                logInfo(m18n('removed stale socket <filename>%1</filename>', socketName()))
+            msg += '\n\n\n' + m18n('Please try again')
         self.dlg = None
-        logWarning(msg)
-        return failure
+        if msg:
+            logWarning(msg)
+        raise CancelledError
 
     def pingLater(self, dummyResult=None):
         """ping the server every 5 seconds"""
