@@ -1,8 +1,25 @@
-# Copyright (c) 2001-2011 Twisted Matrix Laboratories.
-# See LICENSE for details.
-
+# -*- coding: utf-8 -*-
 # pylint: skip-file
 
+"""
+Copyright (c) 2001-2011 Twisted Matrix Laboratories <twisted-python@twistedmatrix.com>
+Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+
+Kajongg is free software you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+"""
+
 """
 This module provides support for Twisted to be driven by the Qt mainloop.
 
@@ -45,12 +62,8 @@ from twisted.python import log, runtime
 from twisted.internet import posixbase
 from twisted.python.runtime import platformType, platform
 
-try:
-    from PyQt4.QtCore import QSocketNotifier, QObject, SIGNAL, QTimer, QCoreApplication
-    from PyQt4.QtCore import QEventLoop
-except ImportError:
-    from PySide.QtCore import QSocketNotifier, QObject, SIGNAL, QTimer, QCoreApplication
-    from PySide.QtCore import QEventLoop
+from qt import QSocketNotifier, QObject, QTimer, QCoreApplication
+from qt import QEventLoop
 
 
 class TwistedSocketNotifier(QObject):
@@ -69,12 +82,12 @@ class TwistedSocketNotifier(QObject):
             self.fn = self.read
         else:
             self.fn = self.write
-        QObject.connect(self.notifier, SIGNAL("activated(int)"), self.fn)
+        self.notifier.activated.connect(self.fn)
 
 
     def shutdown(self):
         self.notifier.setEnabled(False)
-        self.disconnect(self.notifier, SIGNAL("activated(int)"), self.fn)
+        self.notifier.activated.disconnect(self.fn)
         self.fn = self.watcher = None
         self.notifier.deleteLater()
         self.deleteLater()
@@ -99,7 +112,7 @@ class TwistedSocketNotifier(QObject):
             if why:
                 self.reactor._disconnectSelectable(w, why, inRead)
             elif self.watcher:
-                self.notifier.setEnabled(True) # Re enable notification following sucessfull read
+                self.notifier.setEnabled(True) # Re enable notification following successful read
             self.reactor._iterate(fromqt=True)
         log.callWithLogger(w, _read)
 
@@ -134,7 +147,7 @@ class QtReactor(posixbase.PosixReactorBase):
         self._notifiers = {}
         self._timer = QTimer()
         self._timer.setSingleShot(True)
-        QObject.connect(self._timer, SIGNAL("timeout()"), self.iterate)
+        self._timer.timeout.connect(self.iterate)
 
         if QCoreApplication.instance() is None:
             # Application Object has not been started yet
@@ -349,8 +362,6 @@ def win32install():
 
 
 if runtime.platform.getType() == 'win32':
-    from win32event import CreateEvent, MsgWaitForMultipleObjects
-    from win32event import WAIT_OBJECT_0, WAIT_TIMEOUT, QS_ALLINPUT, QS_ALLEVENTS
     install = win32install
 else:
     install = posixinstall
